from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.map_detail import MapDetail
from ...types import Response


def _get_kwargs(
    ids: str,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/maps/ids/{ids}",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, MapDetail]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = MapDetail.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, MapDetail]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    ids: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[Any, MapDetail]]:
    """Get maps for mapIds

    Args:
        ids (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, MapDetail]]
    """

    kwargs = _get_kwargs(
        ids=ids,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    ids: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[Any, MapDetail]]:
    """Get maps for mapIds

    Args:
        ids (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, MapDetail]
    """

    return sync_detailed(
        ids=ids,
        client=client,
    ).parsed


async def asyncio_detailed(
    ids: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[Any, MapDetail]]:
    """Get maps for mapIds

    Args:
        ids (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, MapDetail]]
    """

    kwargs = _get_kwargs(
        ids=ids,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    ids: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[Any, MapDetail]]:
    """Get maps for mapIds

    Args:
        ids (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, MapDetail]
    """

    return (
        await asyncio_detailed(
            ids=ids,
            client=client,
        )
    ).parsed
